;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         DIMMER - EX7                            *
;*                       DESBRAVANDO O PIC                         *
;*       DESENVOLVIDO PELA MOSAICO ENGENHARIA E CONSULTORIA        *
;*      VERSO: 1.0                             DATA: 02/12/99     *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                      DESCRIO DO ARQUIVO                       *
;*-----------------------------------------------------------------*
;*                                                                 *
;* EXEMPLO DO PROJETO PROPOSTO DO DIMMER PARA REGULAGEM POR PWM    *
;* A BASE DE TEMPO DO PWM SER DADA POR INTERRUPO DE TMR0, A     *
;* QUAL OCORRE A CADA 1MS.                                         *
;* A VARIVEL "PERIODO" IR CONTROLAR O PWM, JUNTAMENTE COM A      *
;* VARIVEL "CONTADOR", QUE CONTROLA O DUTY CICLE.                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ARQUIVOS DE DEFINIES                      *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#INCLUDE <P16F84.INC>		;ARQUIVO PADRO MICROCHIP PARA 16F84

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    PAGINAO DE MEMRIA                         *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;DEFINIO DE COMANDOS DE USURIO PARA ALTERAO DA PGINA DE MEMRIA

#DEFINE	BANK0	BCF STATUS,RP0	;SETA BANK 0 DE MEMRIA
#DEFINE	BANK1	BSF STATUS,RP0	;SETA BANK 1 DE MAMRIA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         VARIVEIS                               *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DOS NOMES E ENDEREOS DE TODAS AS VARIVEIS UTILIZADAS 
; PELO SISTEMA

	CBLOCK	0x0C		;ENDEREO INICIAL DA MEMRIA DE
				;USURIO

		W_TEMP		;REGISTRADORES TEMPORRIOS PARA
		STATUS_TEMP	;INTERRUPES
				;ESTAS VARIVEIS NEM SERO UTI-
				;LIZADAS
		CONTADOR	;ARMAZENA O VALOR DA CONTAGEM
		FLAGS		;ARMAZENA OS FLAGS DE CONTROLE
		FILTRO1		;FILTRAGEM PARA O BOTO 1
		FILTRO2		;FILTRAGEM PARA O BOTO 2
                PERIODO         ;CONTROLA O PWM
	ENDC			;FIM DO BLOCO DE MEMRIA		

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                        FLAGS INTERNOS                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS FLAGS UTILIZADOS PELO SISTEMA

#DEFINE	ST_BT1	FLAGS,0		;STATUS DO BOTO 1
#DEFINE	ST_BT2	FLAGS,1		;STATUS DO BOTO 2

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         CONSTANTES                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODAS AS CONSTANTES UTILIZADAS PELO SISTEMA

MIN		EQU	.0	;VALOR MNIMO PARA O CONTADOR
MAX		EQU	.15	;VALOR MXIMO PARA O CONTADOR
T_FILTRO	EQU	.255	;FILTRO PARA BOTO
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           ENTRADAS                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO ENTRADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE	BOTAO1	PORTA,1	;PORTA DO BOTO
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

#DEFINE	BOTAO2	PORTA,2	;PORTA DO BOTO
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           SADAS                                *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO SADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE LED     PORTA,0 ;LED PARA CONTADOR > 15

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       VETOR DE RESET                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	ORG	0x00	;ENDEREO INICIAL DE PROCESSAMENTO
	GOTO	INICIO
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    INCIO DA INTERRUPO                        *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; AS INTERRUPES NO SERO UTILIZADAS, POR ISSO PODEMOS SUBSTITUIR
; TODO O SISTEMA EXISTENTE NO ARQUIVO MODELO PELO APRESENTADO ABAIXO
; ESTE SISTEMA NO  OBRIGATRIO, MAS PODE EVITAR PROBLEMAS FUTUROS

	ORG	0x04		;ENDEREO INICIAL DA INTERRUPO

        MOVWF   W_TEMP          ;COPIA W PARA W_TEMP
        SWAPF   STATUS,W
        MOVWF   STATUS_TEMP     ;COPIA STATUS PARA STATUS_TEMP

        BTFSS   INTCON,T0IF	;FOI INTERRUPO DE TIMER?
        GOTO    SAI_INT		;NO, SAI SEM AO
				;SIM, ENTO PASSOU 1MS
        BCF     INTCON,T0IF	;LIMPA FLAG DA INTERRUPO
        MOVLW   .131    
        MOVWF   TMR0            ;REINICIALIZA TMR0 PARA PRXIMA
        
        MOVLW   .15
        XORWF   PERIODO,W	
        BTFSC   STATUS,Z	;PERODO = 15?
        CLRF    PERIODO		;SIM, ENTO PERIDO = 0
				;NO
        MOVF    PERIODO,F
        BTFSC   STATUS,Z	;PERDODO = 0?
        BSF     LED		;SIM, ENTO LIGA A SADA
                		;NO
        MOVF    PERIODO,W	
        XORWF   CONTADOR,W
        BTFSC   STATUS,Z	;PERODO = CONTADOR?
        BCF     LED		;SIM, DESLIGA A SADA
        			;NO
        INCF    PERIODO,F	;INCREMENTA PERODO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       FIM DA INTERRUPO                        *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
SAI_INT
        SWAPF   STATUS_TEMP,W	;COPIA STATUS_TEMP PARA STATUS
        MOVWF   STATUS
        SWAPF   W_TEMP,F	;COPIA W_TEMP PARA W
        SWAPF   W_TEMP,W
	RETFIE			;RETORNA DA INTERRUPO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                 ROTINA DE CONVERSO BINRIO -> DISPLAY          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA IR RETORNAR EM W, O SIMBOLO CORRETO QUE DEVE SER
; MOSTRADO NO DISPLAY PARA CADA VALOR DE CONTADOR. O RETORNO J EST
; FORMATADO PARA AS CONDIES DE LIGAO DO DISPLAY AO PORTB.
;	   a
;     **********
;     *        *
;   f *        * b
;     *    g   *
;     **********
;     *        *
;   e *        * c
;     *    d   *
;     **********  *.

CONVERTE
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
	ANDLW	B'00001111'	;MASCARA VALOR DE CONTADOR
				;CONSIDERAR SOMENTE AT 15
	ADDWF	PCL,F

;		B'EDC.BAFG'	; POSIO CORRETA DOS SEGUIMENTOS
	RETLW	B'11101110'	; 00 - RETORNA SMBOLO CORRETO 0
	RETLW	B'00101000'	; 01 - RETORNA SMBOLO CORRETO 1
	RETLW	B'11001101'	; 02 - RETORNA SMBOLO CORRETO 2
	RETLW	B'01101101'	; 03 - RETORNA SMBOLO CORRETO 3
	RETLW	B'00101011'	; 04 - RETORNA SMBOLO CORRETO 4	
	RETLW	B'01100111'	; 05 - RETORNA SMBOLO CORRETO 5	
	RETLW	B'11100111'	; 06 - RETORNA SMBOLO CORRETO 6	
	RETLW	B'00101100'	; 07 - RETORNA SMBOLO CORRETO 7	
	RETLW	B'11101111'	; 08 - RETORNA SMBOLO CORRETO 8	
	RETLW	B'01101111'	; 09 - RETORNA SMBOLO CORRETO 9	
	RETLW	B'10101111'	; 10 - RETORNA SMBOLO CORRETO A	
	RETLW	B'11100011'	; 11 - RETORNA SMBOLO CORRETO b	
	RETLW	B'11000110'	; 12 - RETORNA SMBOLO CORRETO C	
	RETLW	B'11101001'	; 13 - RETORNA SMBOLO CORRETO d	
	RETLW	B'11000111'	; 14 - RETORNA SMBOLO CORRETO E	
	RETLW	B'10000111'	; 15 - RETORNA SMBOLO CORRETO F	

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIO DO PROGRAMA                          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
INICIO
	BANK1			;ALTERA PARA O BANCO 1
	MOVLW	B'00000110'
	MOVWF	TRISA		;DEFINE RA1 E 2 COMO ENTRADA E DEMAIS
				;COMO SADAS
	MOVLW	B'00000000'
	MOVWF	TRISB		;DEFINE TODO O PORTB COMO SADA
	MOVLW	B'10000010'
	MOVWF	OPTION_REG	;PRESCALER 1:8 NO TMR0
				;PULL-UPS DESABILITADOS
				;AS DEMAIS CONFG. SO IRRELEVANTES
	MOVLW	B'10100000'
	MOVWF	INTCON		;TODAS AS INTERRUPES DESLIGADAS
	BANK0			;RETORNA PARA O BANCO 0

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIALIZAO DAS VARIVEIS                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
        
        CLRF	PORTA		;LIMPA O PORTA
	CLRF	PORTB		;LIMPA O PORTB
	MOVLW   .131
	MOVWF   TMR0	   	;INICIALIZA TMR0 (256-125)
	CLRF	PERIODO		;INICIALIZA PERODO = 0
	CLRF	FLAGS		;LIMPA TODOS OS FLAGS
	MOVLW	MIN
	MOVWF	CONTADOR	;INICIA CONTADOR = MIN
	GOTO	ATUALIZA	;ATUALIZA O DISPLAY INICIALMENTE

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ROTINA PRINCIPAL                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
MAIN
	MOVLW	T_FILTRO
	MOVWF	FILTRO1		;INICIALIZA FILTRO1 = T_FILTRO
	MOVWF	FILTRO2		;INICIALIZA FILTRO2 = T_FILTRO

CHECA_BT1
	BTFSC	BOTAO1		;O BOTO 1 EST PRESSIONADO?
	GOTO	BT1_LIB		;NO, ENTO TRATA COMO LIBERADO
				;SIM
	DECFSZ	FILTRO1,F	;DECREMENTA O FILTRO DO BOTO
				;TERMINOU?
	GOTO	CHECA_BT1	;NO, CONTINUA ESPERANDO
				;SIM
	BTFSS	ST_BT1		;BOTO J ESTAVA PRESSIONADO?
	GOTO	DEC		;NO, EXECUTA AO DO BOTO
	GOTO	CHECA_BT2	;SIM, CHECA BOTO 2

BT1_LIB
	BCF	ST_BT1		;MARCA BOTO 1 COMO LIBERADO

CHECA_BT2
	BTFSC	BOTAO2		;O BOTO 2 EST PRESSIONADO?
	GOTO	BT2_LIB		;NO, ENTO TRATA COMO LIBERADO
				;SIM
	DECFSZ	FILTRO2,F	;DECREMENTA O FILTRO DO BOTO
				;TERMINOU?
	GOTO	CHECA_BT2	;NO, CONTINUA ESPERANDO
				;SIM
	BTFSS	ST_BT2		;BOTO J ESTAVA PRESSIONADO?
	GOTO	INC		;NO, EXECUTA AO DO BOTO
	GOTO	MAIN		;SIM, VOLTA AO LOOPING

BT2_LIB
	BCF	ST_BT2		;MARCA BOTO 2 COMO LIBERADO
	GOTO	MAIN		;RETORNA AO LOOPING

DEC				;AO DE DECREMENTAR
	BSF	ST_BT1		;MARCA BOTO 1 COMO J PRESSIONADO
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
	XORLW	MIN		;APLICA XOR ENTRE CONTADOR E MIN
				;PARA TESTAR IGUALDADE. SE FOREM
				;IGUAIS, O RESULTADO SER ZERO
	BTFSC	STATUS,Z	;RESULTOU EM ZERO?
	GOTO	MAIN		;SIM, RETORNA SEM AFETAR CONT.
				;NO
	DECF	CONTADOR,F	;DECREMENTA O CONTADOR
	GOTO	ATUALIZA	;ATUALIZA O DISPLAY

INC				;AO DE INCREMENTAR
	BSF	ST_BT2		;MARCA BOTO 2 COMO J PRESSIONADO
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
	XORLW	MAX		;APLICA XOR ENTRE CONTADOR E MAX
				;PARA TESTAR IGUALDADE. SE FOREM
				;IGUAIS, O RESULTADO SER ZERO
	BTFSC	STATUS,Z	;RESULTOU EM ZERO?
	GOTO	MAIN		;SIM, RETORN SEM AFETAR CONT.
				;NO
	INCF	CONTADOR,F	;INCREMENTA O CONTADOR
	GOTO	ATUALIZA	;ATUALIZA O DISPLAY

ATUALIZA
	CALL	CONVERTE	;CONVERTE CONTADOR NO NMERO DO
				;DISPLAY
	MOVWF	PORTB		;ATUALIZA O PORTB PARA
				;VISUALIZARMOS O VALOR DE CONTADOR
				;NO DISPLAY
	GOTO	MAIN		;NO, VOLTA AO LOOP PRINCIPAL
        
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       FIM DO PROGRAMA                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	END			;OBRIGATRIO

