;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       PISCA-PISCA - EX3                         *
;*                       DESBRAVANDO O PIC                         *
;*       DESENVOLVIDO PELA MOSAICO ENGENHARIA E CONSULTORIA        *
;*      VERSO: 1.0                             DATA: 11/06/99     *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                      DESCRIO DO ARQUIVO                       *
;*-----------------------------------------------------------------*
;*  PISCA-PISCA VARIVEL PARA DEMONSTRAR A IMPLEMENTAO DE        *
;*  DELAYS E A INVERSO DE PORTAS.                                 *
;*                                                                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ARQUIVOS DE DEFINIES                      *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#INCLUDE <P16F84.INC>		;ARQUIVO PADRO MICROCHIP PARA 16F84

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    PAGINAO DE MEMRIA                         *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;DEFINIO DE COMANDOS DE USURIO PARA ALTERAO DA PGINA DE MEMRIA

#DEFINE	BANK0	BCF STATUS,RP0	;SETA BANK 0 DE MEMRIA
#DEFINE	BANK1	BSF STATUS,RP0	;SETA BANK 1 DE MAMRIA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         VARIVEIS                               *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DOS NOMES E ENDEREOS DE TODAS AS VARIVEIS UTILIZADAS 
; PELO SISTEMA

	CBLOCK	0x0C		;ENDEREO INICIAL DA MEMRIA DE
				;USURIO

		W_TEMP		;REGISTRADORES TEMPORRIOS PARA
		STATUS_TEMP	;INTERRUPES
				;ESTAS VARIVEIS NEM SERO UTI-
				;LIZADAS
		CONTADOR	;BASE DE TEMPO PARA A PISCADA
		FILTRO		;FILTRAGEM PARA O BOTO
		TEMPO1		;REGISTRADORES AUXILIARES DE TEMPO
		TEMPO2
		TEMPO3

	ENDC			;FIM DO BLOCO DE MEMRIA		

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                        FLAGS INTERNOS                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS FLAGS UTILIZADOS PELO SISTEMA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         CONSTANTES                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODAS AS CONSTANTES UTILIZADAS PELO SISTEMA

MIN		EQU	.10
MAX		EQU	.240
STEP		EQU	.5
MULTIPLO	EQU	.5

;A CONSTANTE DISPLAY REPRESENTA O SMBOLO QUE APARECER PISCANDO NO
;DISPLAY. 1=LED LIGADO E 0=LED DESLIGADO. A RELAO ENTRE BITS E
;SEGMENTOS  A SEGUINTE: 'EDC.BAFG'
;	   a
;     **********
;     *        *
;   f *        * b
;     *    g   *
;     **********
;     *        *
;   e *        * c
;     *    d   *
;     **********  *.

DISPLAY		EQU	B'10101011' ;(LETRA H)
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           ENTRADAS                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO ENTRADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE	BT1	PORTA,1	;BOTO 1 - INCREMENTA
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

#DEFINE	BT2	PORTA,2	;BOTO 2 - DECREMENTA
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           SADAS                                *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO SADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)


;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       VETOR DE RESET                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	ORG	0x00	;ENDEREO INICIAL DE PROCESSAMENTO
	GOTO	INICIO
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    INCIO DA INTERRUPO                        *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; AS INTERRUPES NO SERO UTILIZADAS, POR ISSO PODEMOS SUBSTITUIR
; TODO O SISTEMA EXISTENTE NO ARQUIVO MODELO PELO APRESENTADO ABAIXO
; ESTE SISTEMA NO  OBRIGATRIO, MAS PODE EVITAR PROBLEMAS FUTUROS

	ORG	0x04		;ENDEREO INICIAL DA INTERRUPO
	RETFIE			;RETORNA DA INTERRUPO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                        ROTINA DE DELAY                          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA AGUARDA TANTOS MILISEGUNDOS QUANTO O VALOR PASSADO
; POR W. POR EXEMPLO, SE W = .200, ELA AGUARDAR 200 MILISEGUNDOS.
;
; O DELAY PRINCIPAL DURA 1ms, POIS POSSUI 5 INSTRUES (5us) E 
; RODADO 200 VEZES (TEMPO1). PORTANTO 200 * 5us = 1ms.
; O DELAY PRINCIPAL  RODADO TANTAS VEZES QUANTO FOR O VALOR DE
; TEMPO2, O QUAL  INICIADO COM O VALOR PASSADO EM W.


DELAY
	MOVWF	TEMPO2		;INICIA TEMPO 2 COM O VALOR
				;PASSADO EM W
DL1	
	MOVLW	.200
	MOVWF	TEMPO1

DL2				;ESTE DELAY DURA 1ms (5*200)
	NOP
	NOP
	DECFSZ	TEMPO1,F	;DECREMENTA TEMPO1. ACABOU?
	GOTO	DL2		;NO, CONTINUA AGUARDANDO
				;SIM

	DECFSZ	TEMPO2,F	;DECREMENTA TEMPO2. ACABOU?
	GOTO	DL1		;NO, CONTINUA AGUARDANDO
				;SIM
	RETURN

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIO DO PROGRAMA                          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
INICIO
	BANK1			;ALTERA PARA O BANCO 1
	MOVLW	B'00000110'
	MOVWF	TRISA		;DEFINE RA1 E 2 COMO ENTRADA E DEMAIS
				;COMO SADAS
	MOVLW	B'00000000'
	MOVWF	TRISB		;DEFINE TODO O PORTB COMO SADA
	MOVLW	B'10000000'
	MOVWF	OPTION_REG	;PRESCALER 1:2 NO TMR0
				;PULL-UPS DESABILITADOS
				;AS DEMAIS CONFG. SO IRRELEVANTES
	MOVLW	B'00000000'
	MOVWF	INTCON		;TODAS AS INTERRUPES DESLIGADAS
	BANK0			;RETORNA PARA O BANCO 0

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIALIZAO DAS VARIVEIS                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	CLRF	PORTA		;LIMPA O PORTA
	MOVLW	DISPLAY
	MOVWF	PORTB		;ACENDE O VALOR CERTO NO DISPLAY
	MOVLW	MIN
	MOVWF	CONTADOR	;INICIA CONTADOR COM VALOR MIN.


;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ROTINA PRINCIPAL                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
MAIN
	MOVLW	MULTIPLO
	MOVWF	TEMPO3		;INICIA COMTADOR DE MULTIPLICAO,
				;POIS OS TEMPOS GERADOS POR DELAY
				;SO MUITO PEQUENOS, GERANDO FREQ.
				;MUITO ALTAS PARA A VISUALIZAO.
MAIN1
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
				;PARA CHAMAR A ROTINA DE DELAY
	CALL	DELAY		;CHAMA ROTINA DE DELAY

	BTFSS	BT1		;BOTO 1 PRESSIONADO?
	GOTO	INCREMENTA	;SIM, DEVE INCREMENTAR
				;NO

	BTFSS	BT2		;BOTO 2 PRESSIONADO?
	GOTO	DECREMENTA	;SIM, DEVE DECREMENTAR
				;NO

	DECFSZ	TEMPO3,F	;DECREMENTA CONTADOR DE MULT. ACABOU?
	GOTO	MAIN1		;NO, CONTINUA AGUARDANDO
				;SIM	

	MOVLW	DISPLAY		;APS TRANSCORRIDO O TEMPO, IR
				;INVERTER OS LEDS CORRETOS ATRAVS
				;DA MSCARA "DISPLAY" E DA OPERAO
				;XOR
	XORWF	PORTB,F		;INVERTE LEDS -> PISCA

	GOTO	MAIN		;COMEA NOVAMENTE


DECREMENTA
	MOVLW	STEP
	SUBWF	CONTADOR,F	;DECREMENTA O CONTADOR EM STEP

	MOVLW	MIN		;MOVE O VALOR MNIMO PARA W
	SUBWF	CONTADOR,W	;SUBTRAI O VALOR DE W (MIN) DE CONTADOR
	BTFSC	STATUS,C	;TESTA CARRY. RESULTADO NEGATIVO?
	GOTO	MAIN		;NO, ENTO CONTA >= MIN
				;SIM, ENTO CONTA < MIN

	MOVLW	MIN
	MOVWF	CONTADOR	;ACERTA CONTADOR NO MNIMO, POIS
				;PASSOU DO VALOR

	BTFSS	BT2		;BOTO 2 CONTINUA PRESSIONADO?
	GOTO	$-1		;SIM, AGUARDA LIBERAO
				;NO
	GOTO	MAIN		;VOLTA AO LOOP PRINCIPAL

INCREMENTA
	MOVLW	STEP
	ADDWF	CONTADOR,F	;INCREMENTA O CONTADOR EM STEP

	MOVLW	MAX		;MOVE O VALOR MXIMO PARA W
	SUBWF	CONTADOR,W	;SUBTRAI O VALOR DE W (MIN) DE CONTADOR
	BTFSS	STATUS,C	;TESTA CARRY. RESULTADO NEGATIVO?
	GOTO	MAIN		;SIM, ENTO CONTA < MAX
				;NO, ENTO CONTA >= MAX
	MOVLW	MAX
	MOVWF	CONTADOR	;ACERTA CONTADOR NO MXIMO, POIS
				;PASSOU DO VALOR
	BTFSS	BT1		;BOTO 1 CONTINUA PRESSIONADO?
	GOTO	$-1		;SIM, AGUARDA LIBERAO
				;NO
	GOTO	MAIN		;VOLTA AO LOOP PRINCIPAL


;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       FIM DO PROGRAMA                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	END			;OBRIGATRIO

