;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    TIMER DE SEGUNDOS - EX5                      *
;*                       DESBRAVANDO O PIC                         *
;*       DESENVOLVIDO PELA MOSAICO ENGENHARIA E CONSULTORIA        *
;*      VERSO: 1.0                             DATA: 11/06/99     *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                      DESCRIO DO ARQUIVO                       *
;*-----------------------------------------------------------------*
;*  TIMER DECRESCENTE EM SEGUNDOS. O VALOR INICIAL  DETERMINADO   *
;*  PELA CONSTANTE V_INICIO E PODE ESTAR ENTRE 1 E 9 SEGUNDOS.     *
;*  O BOTO 1 DISPARA O TIMER, MOSTRANDO O TEMPO RESTANTE NO       *
;*  DISPLAY. O BOTO 2 PARALIZA O TIMER. O LED  UTILIZADO PARA    *
;* INDICAR O ESTADO ATUAL DO TIMER: ACESO=RODANDO E APAGADO=PARADO *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ARQUIVOS DE DEFINIES                      *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#INCLUDE <P16F84.INC>		;ARQUIVO PADRO MICROCHIP PARA 16F84

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    PAGINAO DE MEMRIA                         *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;DEFINIO DE COMANDOS DE USURIO PARA ALTERAO DA PGINA DE MEMRIA

#DEFINE	BANK0	BCF STATUS,RP0	;SETA BANK 0 DE MEMRIA
#DEFINE	BANK1	BSF STATUS,RP0	;SETA BANK 1 DE MAMRIA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         VARIVEIS                               *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DOS NOMES E ENDEREOS DE TODAS AS VARIVEIS UTILIZADAS 
; PELO SISTEMA

	CBLOCK	0x0C		;ENDEREO INICIAL DA MEMRIA DE
				;USURIO

		W_TEMP		;REGISTRADORES TEMPORRIOS PARA
		STATUS_TEMP	;INTERRUPES
				;ESTAS VARIVEIS NEM SERO UTI-
				;LIZADAS
		TEMPO		;ARMAZENA O VALOR DO TEMPO
		FLAGS		;ARMAZENA OS FLAGS DE CONTROLE
		TEMP1		;REGISTRADORES AUXILIARES
		TEMP2
		FILTRO1		;FILTROS DOS BOTES
		FILTRO2	

	ENDC			;FIM DO BLOCO DE MEMRIA		

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                        FLAGS INTERNOS                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS FLAGS UTILIZADOS PELO SISTEMA

#DEFINE	F_FIM	FLAGS,0		;FLAG DE FIM DE TEMPO
#DEFINE	ST_BT1	FLAGS,1		;STATUS DO BOTO 1
#DEFINE	ST_BT2	FLAGS,2		;STATUS DO BOTO 2

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         CONSTANTES                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODAS AS CONSTANTES UTILIZADAS PELO SISTEMA

V_INICIO	EQU	.9	;VALOR INICIAL DO TIMER (1 A 15 SEG.)
T_FILTRO	EQU	.255	;VALOR DO FILTRO DOS BOTES
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           ENTRADAS                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO ENTRADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE	BOTAO1	PORTA,1	;PORTA DO BOTO
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

#DEFINE	BOTAO2	PORTA,2	;PORTA DO BOTO
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           SADAS                                *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO SADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE	LED	PORTA,3	;LED
			;0 -> DESLIGADO
			;1 -> LIGADO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       VETOR DE RESET                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	ORG	0x00	;ENDEREO INICIAL DE PROCESSAMENTO
	GOTO	INICIO
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    INCIO DA INTERRUPO                        *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; UTILIZAREMOS SOMENTE A INTERRUO DE TMR0, MAS EFETUAREMOS O TESTE
; PARA TERMOS CERTEZA DE QUE NENHUM PROBLEMA ACONTECEU.  NECESSRIO
; SALVAR E RECUPERAR OS VALOR DE W E STATUS.

	ORG	0x04		;ENDEREO INICIAL DA INTERRUPO
	MOVWF	W_TEMP		;SALVA W EM W_TEMP
	SWAPF	STATUS,W	
	MOVWF	STATUS_TEMP	;SALVA STATUS EM STATUS_TEMP

	BTFSS	INTCON,T0IF	; INTERRUPO DE TMR0?
	GOTO	SAI_INT		;NO, SAI SE AO	
				;SIM

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*              TRATAMENTO DA INTERRUPO DE TMR0                  *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA IR CONTAR O TEMPO, E QUANDO PASSAR 1 SEGUNDO, A VARI-
; VEL "TEMPO" SER DECREMENTADA.
; 1 SEGUNDO = 64us (PRESCALER) X 125 (TMR0) X 125 (TEMP1)

	BCF	INTCON,T0IF	;LIMPA FLAG DA INT.
	MOVLW	.256-.125	
	MOVWF	TMR0		;REINICIA TMR0
	DECFSZ	TEMP1,F		;DECREMENTA CONTADOR AUXILIAR. ACABOU?
	GOTO	SAI_INT		;NO, SAI SEM AO
				;SIM
	MOVLW	.125
	MOVWF	TEMP1		;REINICIALIZA TEMPO AUXILIAR
	BTFSC	F_FIM		;J CHEGOU AO FIM?
	GOTO	SAI_INT		;SIM, ENTO NO DECREMENTA O TEMPO
				;NO
	DECFSZ	TEMPO,F		;DECREMENTA TEMPO. ACABOU?
	GOTO	SAI_INT		;NO, SAI DA INTERRUO
				;SIM
	BSF	F_FIM		;MARCA FIM DO TEMPO
	GOTO	SAI_INT		;SAI DA INTERRUPO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       FIM DA INTERRUPO                        *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
SAI_INT
	SWAPF	STATUS_TEMP,W
	MOVWF	STATUS		;RECUPERA STATUS
	SWAPF	W_TEMP,F
	SWAPF	W_TEMP,W	;RECUPERA W
	RETFIE			;RETORNA DA INTERRUPO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                 ROTINA DE CONVERSO BINRIO -> DISPLAY          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA IR RETORNAR EM W, O SIMBOLO CORRETO QUE DEVE SER
; MOSTRADO NO DISPLAY PARA CADA VALOR DE CONTADOR. O RETORNO J EST
; FORMATADO PARA AS CONDIES DE LIGAO DO DISPLAY AO PORTB.
;	   a
;     **********
;     *        *
;   f *        * b
;     *    g   *
;     **********
;     *        *
;   e *        * c
;     *    d   *
;     **********  *.

CONVERTE
	MOVF	TEMPO,W		;COLOCA CONTADOR EM W
	ANDLW	B'00001111'	;MASCARA VALOR DE CONTADOR
				;CONSIDERAR SOMENTE AT 15
	ADDWF	PCL,F

;		B'EDC.BAFG'	; POSIO CORRETA DOS SEGUIMENTOS
	RETLW	B'11101110'	; 00 - RETORNA SMBOLO CORRETO 0
	RETLW	B'00101000'	; 01 - RETORNA SMBOLO CORRETO 1
	RETLW	B'11001101'	; 02 - RETORNA SMBOLO CORRETO 2
	RETLW	B'01101101'	; 03 - RETORNA SMBOLO CORRETO 3
	RETLW	B'00101011'	; 04 - RETORNA SMBOLO CORRETO 4	
	RETLW	B'01100111'	; 05 - RETORNA SMBOLO CORRETO 5	
	RETLW	B'11100111'	; 06 - RETORNA SMBOLO CORRETO 6	
	RETLW	B'00101100'	; 07 - RETORNA SMBOLO CORRETO 7	
	RETLW	B'11101111'	; 08 - RETORNA SMBOLO CORRETO 8	
	RETLW	B'01101111'	; 09 - RETORNA SMBOLO CORRETO 9	
	RETLW	B'10101111'	; 10 - RETORNA SMBOLO CORRETO A	
	RETLW	B'11100011'	; 11 - RETORNA SMBOLO CORRETO b	
	RETLW	B'11000110'	; 12 - RETORNA SMBOLO CORRETO C	
	RETLW	B'11101001'	; 13 - RETORNA SMBOLO CORRETO d	
	RETLW	B'11000111'	; 14 - RETORNA SMBOLO CORRETO E	
	RETLW	B'10000111'	; 15 - RETORNA SMBOLO CORRETO F	

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                 ROTINA DE ATUALIZAO DO DISPLAY                *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA CONVERTE O VALOR DE TEMPO ATRAVS DA ROTINA CONVERTE
; E ATUALIZA O PORTB PARA ACENDER O DISPLAY CORRETAMENTE

ATUALIZA
	CALL	CONVERTE	;CONVERTE CONTADOR NO NMERO DO
				;DISPLAY
	MOVWF	PORTB		;ATUALIZA O PORTB PARA
				;VISUALIZARMOS O VALOR DE CONTADOR
				;NO DISPLAY
	RETURN			;NO, RETORNA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                 ROTINA DE DESLIGAR O TIMER                      *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA EXECUTA AS AES NECESSRIAS PARA DESLIGAR O TIMER

DESL_TIMER
	BCF	INTCON,GIE	;DESLIGA CHAVE GERAL DE INT.
	BCF	LED		;APAGA O LED
	RETURN			;RETORNA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                 ROTINA DE LIGAR O TIMER                         *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA EXECUTA AS AES NECESSRIAS PARA LIGAR O TIMER

LIGA_TIMER
	BTFSC	INTCON,GIE	;TIMER J ESTA LIGADO?
	RETURN			;SIM, RETORNA DIRETO
				;NO
	BCF	INTCON,T0IF	;LIMPA FLAG DE INT. DE TMR0
	MOVLW	.256-.125
	MOVWF	TMR0		;INICIA TMR0 CORRETAMENTE
	MOVLW	.125
	MOVWF	TEMP1		;INICIA TEMP1 CORRETAMENTE
	BSF	INTCON,GIE	;LIGA CHAVE GERAL DE INTERRUPES
	BSF	LED		;ACENDE O LED
	RETURN			;RETORNA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIO DO PROGRAMA                          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
INICIO
	BANK1			;ALTERA PARA O BANCO 1
	MOVLW	B'00000110'
	MOVWF	TRISA		;DEFINE RA1 E 2 COMO ENTRADA E DEMAIS
				;COMO SADAS
	MOVLW	B'00000000'
	MOVWF	TRISB		;DEFINE TODO O PORTB COMO SADA
	MOVLW	B'10000101'
	MOVWF	OPTION_REG	;PRESCALER 1:64 NO TMR0
				;PULL-UPS DESABILITADOS
				;AS DEMAIS CONFG. SO IRRELEVANTES
	MOVLW	B'00100000'
	MOVWF	INTCON		;HABILITADA SOMENTE A INTERRUPO TMR0
				;CHAVE GERAL DAS INTERRUES DESLIGADAS
	BANK0			;RETORNA PARA O BANCO 0

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIALIZAO DAS VARIVEIS                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	CLRF	PORTA		;LIMPA O PORTA
	CLRF	PORTB		;LIMPA O PORTB
	CLRF	FLAGS		;LIMPA TODOS OS FLAGS
	MOVLW	V_INICIO
	MOVWF	TEMPO		;INICIA TEMPO = V_INICIO
	CALL	ATUALIZA	;ATUALIZA O DISPLAY INICIALMENTE

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ROTINA PRINCIPAL                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
MAIN
	BTFSC	F_FIM		;CHEGOU AO FIM?
	CALL	DESL_TIMER	;SIM, ENTO DESLIGA O TIMER
				;NO
	CALL	ATUALIZA	;ATUALIZA O DISPLAY
	MOVLW	T_FILTRO
	MOVWF	FILTRO1		;INICIALIZA FILTRO1 = T_FILTRO
	MOVWF	FILTRO2		;INICIALIZA FILTRO2 = T_FILTRO

CHECA_BT1
	BTFSC	BOTAO1		;O BOTO 1 EST PRESSIONADO?
	GOTO	BT1_LIB		;NO, ENTO TRATA COMO LIBERADO
				;SIM
	DECFSZ	FILTRO1,F	;DECREMENTA O FILTRO DO BOTO
				;TERMINOU?
	GOTO	CHECA_BT1	;NO, CONTINUA ESPERANDO
				;SIM
	BTFSS	ST_BT1		;BOTO J ESTAVA PRESSIONADO?
	GOTO	ACAO_BT1	;NO, EXECUTA AO DO BOTO
	GOTO	CHECA_BT2	;SIM, CHECA BOTO 2

BT1_LIB
	BCF	ST_BT1		;MARCA BOTO 1 COMO LIBERADO

CHECA_BT2
	BTFSC	BOTAO2		;O BOTO 2 EST PRESSIONADO?
	GOTO	BT2_LIB		;NO, ENTO TRATA COMO LIBERADO
				;SIM
	DECFSZ	FILTRO2,F	;DECREMENTA O FILTRO DO BOTO
				;TERMINOU?
	GOTO	CHECA_BT2	;NO, CONTINUA ESPERANDO
				;SIM
	BTFSS	ST_BT2		;BOTO J ESTAVA PRESSIONADO?
	GOTO	ACAO_BT2	;NO, EXECUTA AO DO BOTO
	GOTO	MAIN		;SIM, VOLTA AO LOOPING

BT2_LIB
	BCF	ST_BT2		;MARCA BOTO 2 COMO LIBERADO
	GOTO	MAIN		;RETORNA AO LOOPING

ACAO_BT1			;AO PARA O BOTO 1
	BSF	ST_BT1		;MARCA BOTO 1 COMO J PRESSIONADO
	CALL	LIGA_TIMER	;LIGA O TIMER
	GOTO	MAIN

ACAO_BT2			;AO PARA O BOTO 2
	BSF	ST_BT2		;MARCA BOTO 2 COMO J PRESSIONADO
	CALL	DESL_TIMER	;DESLIGA O TIMER
	GOTO	MAIN		;NO, VOLTA AO LOOP PRINCIPAL

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       FIM DO PROGRAMA                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	END			;OBRIGATRIO

