;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     CONTADOR FINAL - EX6                        *
;*                       DESBRAVANDO O PIC                         *
;*       DESENVOLVIDO PELA MOSAICO ENGENHARIA E CONSULTORIA        *
;*      VERSO: 1.0                             DATA: 11/06/99     *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                      DESCRIO DO ARQUIVO                       *
;*-----------------------------------------------------------------*
;*  CONTADOR QUE UTILIZA DOIS BOTES PARA INCREMENTAR E DECRE-     *
;*  MENTAR O VALOR CONTROLADO PELA VARIVEL "CONTADOR". ESTA       *
;*  VARIVEL EST LIMITADA PELAS CONSTANTES "MIN" E "MAX".         *
;*  O VALOR DO CONTADOR  MOSTRADO NO DISPLAY E ARMAZENADO NA      *
;*  EEPROM PARA NO SER PERDIDO MESMO NO CASO DE RESET.            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ARQUIVOS DE DEFINIES                      *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#INCLUDE <P16F84.INC>		;ARQUIVO PADRO MICROCHIP PARA 16F84

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    PAGINAO DE MEMRIA                         *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;DEFINIO DE COMANDOS DE USURIO PARA ALTERAO DA PGINA DE MEMRIA

#DEFINE	BANK0	BCF STATUS,RP0	;SETA BANK 0 DE MEMRIA
#DEFINE	BANK1	BSF STATUS,RP0	;SETA BANK 1 DE MAMRIA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         VARIVEIS                               *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DOS NOMES E ENDEREOS DE TODAS AS VARIVEIS UTILIZADAS 
; PELO SISTEMA

	CBLOCK	0x0C		;ENDEREO INICIAL DA MEMRIA DE
				;USURIO

		W_TEMP		;REGISTRADORES TEMPORRIOS PARA
		STATUS_TEMP	;INTERRUPES
				;ESTAS VARIVEIS NEM SERO UTI-
				;LIZADAS
		CONTADOR	;ARMAZENA O VALOR DA CONTAGEM
		FLAGS		;ARMAZENA OS FLAGS DE CONTROLE
		FILTRO1		;FILTRAGEM PARA O BOTO 1
		FILTRO2		;FILTRAGEM PARA O BOTO 2

	ENDC			;FIM DO BLOCO DE MEMRIA		

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                        FLAGS INTERNOS                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS FLAGS UTILIZADOS PELO SISTEMA

#DEFINE	ST_BT1	FLAGS,0		;STATUS DO BOTO 1
#DEFINE	ST_BT2	FLAGS,1		;STATUS DO BOTO 2

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         CONSTANTES                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODAS AS CONSTANTES UTILIZADAS PELO SISTEMA

MIN		EQU	.0	;VALOR MINIMO PARA O CONTADOR
MAX		EQU	.15	;VALOR MXIMO PARA O CONTADOR
T_FILTRO	EQU	.255	;FILTRO PARA BOTO
POS_MEM		EQU	.0	;ENDEREO DA EEPROM ONDE SER
				;ARMAZENADO O VALOR DO CONTADOR
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           ENTRADAS                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO ENTRADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE	BOTAO1	PORTA,1	;PORTA DO BOTO
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

#DEFINE	BOTAO2	PORTA,2	;PORTA DO BOTO
			; 0 -> PRESSIONADO
			; 1 -> LIBERADO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           SADAS                                *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO SADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)


;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    INICIALIZAO DA EEPROM                      *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	ORG	H'2100'+POS_MEM	;INCIO DA EEPROM
	DE	.5		;VALOR INICIAL PARA CONTADOR = 5
				;VIDE DIRETRIZ "DE" NO APNDICE B

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       VETOR DE RESET                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	ORG	0x00	;ENDEREO INICIAL DE PROCESSAMENTO
	GOTO	INICIO
	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    INCIO DA INTERRUPO                        *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; AS INTERRUPES NO SERO UTILIZADAS, POR ISSO PODEMOS SUBSTITUIR
; TODO O SISTEMA EXISTENTE NO ARQUIVO MODELO PELO APRESENTADO ABAIXO
; ESTE SISTEMA NO  OBRIGATRIO, MAS PODE EVITAR PROBLEMAS FUTUROS

	ORG	0x04		;ENDEREO INICIAL DA INTERRUPO
	RETFIE			;RETORNA DA INTERRUPO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                 ROTINA DE CONVERSO BINRIO -> DISPLAY          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA IR RETORNAR EM W, O SIMBOLO CORRETO QUE DEVE SER
; MOSTRADO NO DISPLAY PARA CADA VALOR DE CONTADOR. O RETORNO J EST
; FORMATADO PARA AS CONDIES DE LIGAO DO DISPLAY AO PORTB.
;	   a
;     **********
;     *        *
;   f *        * b
;     *    g   *
;     **********
;     *        *
;   e *        * c
;     *    d   *
;     **********  *.

CONVERTE
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
	ANDLW	B'00001111'	;MASCARA VALOR DE CONTADOR
				;CONSIDERAR SOMENTE AT 15
	ADDWF	PCL,F

;		B'EDC.BAFG'	; POSIO CORRETA DOS SEGUIMENTOS
	RETLW	B'11101110'	; 00 - RETORNA SMBOLO CORRETO 0
	RETLW	B'00101000'	; 01 - RETORNA SMBOLO CORRETO 1
	RETLW	B'11001101'	; 02 - RETORNA SMBOLO CORRETO 2
	RETLW	B'01101101'	; 03 - RETORNA SMBOLO CORRETO 3
	RETLW	B'00101011'	; 04 - RETORNA SMBOLO CORRETO 4	
	RETLW	B'01100111'	; 05 - RETORNA SMBOLO CORRETO 5	
	RETLW	B'11100111'	; 06 - RETORNA SMBOLO CORRETO 6	
	RETLW	B'00101100'	; 07 - RETORNA SMBOLO CORRETO 7	
	RETLW	B'11101111'	; 08 - RETORNA SMBOLO CORRETO 8	
	RETLW	B'01101111'	; 09 - RETORNA SMBOLO CORRETO 9	
	RETLW	B'10101111'	; 10 - RETORNA SMBOLO CORRETO A	
	RETLW	B'11100011'	; 11 - RETORNA SMBOLO CORRETO b	
	RETLW	B'11000110'	; 12 - RETORNA SMBOLO CORRETO C	
	RETLW	B'11101001'	; 13 - RETORNA SMBOLO CORRETO d	
	RETLW	B'11000111'	; 14 - RETORNA SMBOLO CORRETO E	
	RETLW	B'10000111'	; 15 - RETORNA SMBOLO CORRETO F	

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                  ROTINA DE LEITURA NA E2PROM                  *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA L O BYTE DO ENDEREO ACERTADO POR EEADR E COLOCA
; O VALOR EM W.

LE_E2PROM
	BANK1			;BANCO 1
	BSF	EECON1,RD	;PREPARA LEITURA
	BANK0			;BANCO 0
	MOVF	EEDATA,W	;COLOCA DADO EM W
	RETURN			;RETORNA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                  ROTINA DE ESCRITA NA E2PROM                  *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA ESCREVE O DADO PASSSADO EM W NO ENDEREO ACERTADO
; ANTERIORMENTE EM EEADR

ESCR_E2PROM
	MOVWF	EEDATA		;ACERTA DADO PASSADO EM W 
	BCF	INTCON,GIE	;DESABILITA INTERRUPES
	BANK1			;BANCO 1
	BSF	EECON1,WREN	;HABILITA ESCRITA
	MOVLW	0X55		;INICIALIZAO DA ESCRITA
	MOVWF	EECON2
	MOVLW	0XAA
	MOVWF	EECON2
	BSF	EECON1,WR	;INICIA ESCRITA
	BCF	EECON1,WREN
	BTFSC	EECON1,WR	;ACABOU ESCRITA?
	GOTO	$-1		;NO, AGUARDA
	BANK0			;SIM, BANCO 0
	BSF	INTCON,GIE	;HABILITA INTERRUPES
	RETURN			;RETORNA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*             ROTINA DE LEITURA DO VALOR DO CONTADOR              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA L O VALOR DA MEMRIA E COLOCA O RESULTADO NA
; VARIVEL "CONTADOR".

LE_CONTA
	MOVLW	POS_MEM
	MOVWF	EEADR		;ACERTA O ENDEREO PARA LEITURA
	CALL	LE_E2PROM	;EFETUA A LEITURA DA EEPROM
	MOVWF	CONTADOR	;ATUALIZA O CONTADOR
	RETURN			;RETORNA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*             ROTINA DE ESCRITA DO VALOR DO CONTADOR              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; ESTA ROTINA ESCREVE O VALOR ATUAL DE CONTADOR NA MEMRIA EEPROM

ESCR_CONTA
	MOVLW	POS_MEM
	MOVWF	EEADR		;ACERTA O ENDEREO PARA LEITURA
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
	CALL	ESCR_E2PROM	;EFETUA A ESCRITA EEPROM
	RETURN			;RETORNA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIO DO PROGRAMA                          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
INICIO
	BANK1			;ALTERA PARA O BANCO 1
	MOVLW	B'00000110'
	MOVWF	TRISA		;DEFINE RA1 E 2 COMO ENTRADA E DEMAIS
				;COMO SADAS
	MOVLW	B'00000000'
	MOVWF	TRISB		;DEFINE TODO O PORTB COMO SADA
	MOVLW	B'10000000'
	MOVWF	OPTION_REG	;PRESCALER 1:2 NO TMR0
				;PULL-UPS DESABILITADOS
				;AS DEMAIS CONFG. SO IRRELEVANTES
	MOVLW	B'00000000'
	MOVWF	INTCON		;TODAS AS INTERRUPES DESLIGADAS
	BANK0			;RETORNA PARA O BANCO 0

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIALIZAO DAS VARIVEIS                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	CLRF	PORTA		;LIMPA O PORTA
	CLRF	PORTB		;LIMPA O PORTB
	CLRF	FLAGS		;LIMPA TODOS OS FLAGS
	CALL	LE_CONTA	;INICIALIZA CONTADOR COM VALOR
				;DA EEPROM
	GOTO	ATUALIZA	;ATUALIZA O DISPLAY INICIALMENTE

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ROTINA PRINCIPAL                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
MAIN
	MOVLW	T_FILTRO
	MOVWF	FILTRO1		;INICIALIZA FILTRO1 = T_FILTRO
	MOVWF	FILTRO2		;INICIALIZA FILTRO2 = T_FILTRO

CHECA_BT1
	BTFSC	BOTAO1		;O BOTO 1 EST PRESSIONADO?
	GOTO	BT1_LIB		;NO, ENTO TRATA COMO LIBERADO
				;SIM
	DECFSZ	FILTRO1,F	;DECREMENTA O FILTRO DO BOTO
				;TERMINOU?
	GOTO	CHECA_BT1	;NO, CONTINUA ESPERANDO
				;SIM
	BTFSS	ST_BT1		;BOTO J ESTAVA PRESSIONADO?
	GOTO	DEC		;NO, EXECUTA AO DO BOTO
	GOTO	CHECA_BT2	;SIM, CHECA BOTO 2

BT1_LIB
	BCF	ST_BT1		;MARCA BOTO 1 COMO LIBERADO

CHECA_BT2
	BTFSC	BOTAO2		;O BOTO 2 EST PRESSIONADO?
	GOTO	BT2_LIB		;NO, ENTO TRATA COMO LIBERADO
				;SIM
	DECFSZ	FILTRO2,F	;DECREMENTA O FILTRO DO BOTO
				;TERMINOU?
	GOTO	CHECA_BT2	;NO, CONTINUA ESPERANDO
				;SIM
	BTFSS	ST_BT2		;BOTO J ESTAVA PRESSIONADO?
	GOTO	INC		;NO, EXECUTA AO DO BOTO
	GOTO	MAIN		;SIM, VOLTA AO LOOPING

BT2_LIB
	BCF	ST_BT2		;MARCA BOTO 2 COMO LIBERADO
	GOTO	MAIN		;RETORNA AO LOOPING

DEC				;AO DE DECREMENTAR
	BSF	ST_BT1		;MARCA BOTO 1 COMO J PRESSIONADO
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
	XORLW	MIN		;APLICA XOR ENTRE CONTADOR E MIN
				;PARA TESTAR IGUALDADE. SE FOREM
				;IGUAIS, O RESULTADO SER ZERO
	BTFSC	STATUS,Z	;RESULTOU EM ZERO?
	GOTO	MAIN		;SIM, RETORNA SEM AFETAR CONT.
				;NO
	DECF	CONTADOR,F	;DECREMENTA O CONTADOR
	CALL	ESCR_CONTA	;ATUALIZA O VALOR DE CONTADOR NA 
				;EEPROM
	GOTO	ATUALIZA	;ATUALIZA O DISPLAY

INC				;AO DE INCREMENTAR
	BSF	ST_BT2		;MARCA BOTO 2 COMO J PRESSIONADO
	MOVF	CONTADOR,W	;COLOCA CONTADOR EM W
	XORLW	MAX		;APLICA XOR ENTRE CONTADOR E MAX
				;PARA TESTAR IGUALDADE. SE FOREM
				;IGUAIS, O RESULTADO SER ZERO
	BTFSC	STATUS,Z	;RESULTOU EM ZERO?
	GOTO	MAIN		;SIM, RETORNA SEM AFETAR CONT.
				;NO
	INCF	CONTADOR,F	;INCREMENTA O CONTADOR
	CALL	ESCR_CONTA	;ATUALIZA O VALOR DE CONTADOR NA 
				;EEPROM

ATUALIZA			;ATUALIZAO DO DISPLAY
	CALL	CONVERTE	;CONVERTE CONTADOR NO NMERO DO
				;DISPLAY
	MOVWF	PORTB		;ATUALIZA O PORTB PARA
				;VISUALIZARMOS O VALOR DE CONTADOR
				;NO DISPLAY
	GOTO	MAIN		;NO, VOLTA AO LOOP PRINCIPAL

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       FIM DO PROGRAMA                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	END			;OBRIGATRIO

