;=================================================================== 
;	Programa exemplo 4 para o livro "Microcontroladores PIC"
;
;	Objetivo: 		Incrementar/Decrementar posicao de memoria
;				pelas teclas S0 e S1.
;	Funcionamento: 		posicao "valor" varia entre 0 e 63 (00111111)
;				Inicia com valor = 32 = 00100000
; 				S0 -> valor = valor - 1, se valor > 0
; 				S1 -> valor = valor + 1, se valor < 63
;	17/09/98 	Leds agora acendem em 1, em oposicao ao do livro.
;			Adaptacao para a placa PicLab 3
;=================================================================== 
	list p=16F84		; processador 16F84

	__config B'0011111111110001' 
				; CP off, DATA off, WD off, PWT on, XT

	radix	dec		; define padrao DECIMAL para valores sem identificacao
	include	<P16F84.INC>	; anexa o arquivo que contem as definicoes do 16F84
;.....................................................................................
;				Tabela de definicoes de RAM e constantes.
W	equ	0		; referencia de W
S0	equ	0		; tecla S0 esta ligada a RA0 (PORTA bit 0)
S1	equ	1		; tecla S1 esta ligada a RA1 (PORTA bit 1)
tempo	equ	0CH		; define variavel auxiliar tempo na RAM 0CH. (primeira posicao)
dt1	equ	0DH		; variavel auxiliar para tempo
X	equ	0EH		
Y	equ	0FH
valor   equ	10H		; valor ajustado por S0 e S1
;----------------------------- Memoria de Programa -----------------------------------
	org	0		; define inicio do trecho a seguir em 000
	goto	inicio		; desvia para o programa principal

;.............................. INICIO DA ROTINA DE INTERRUPCAO ............................
	org	4		
	nop			; primeira instrucao da interrupcao.
	retfie			; retorna da interrupcao.

;.............................. INICIO DO PROGRAMA .........................................
inicio:				; este endereco e' o imediatamente apos o fim da interrupcao
				; AQUI ESTAMOS NO BANCO 0
	movlw	B'00000000'	; W = 0
	movwf	INTCON	; INTCON = 0. Nenhuma interr habilitada.
	clrf	PORTA		;inicializa portas A e B
	clrf	PORTB

	bsf	STATUS,RP0	; Seleciona banco 1 para option e tris

	movlw	B'11111111'	; W = 255
	movwf	TRISA		; TRISA = ffH. PortA toda entrada
	movlw	B'00000000'	; W = 0
	movwf	TRISB		; TRISB = 0. PortB toda saida
	movlw	B'11011111'	;
	movwf	OPTION_REG	; Registro OPTION ajusta timer 0 clock interno
				; e prescaler p/ Watch Dog (nao usamos)

	bcf	STATUS,RP0	; volto ao banco 0. Padrao do reset

;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
principal:			;inicio do programa principal.
	movlw	32		; W = 00100000
	movwf	valor		; valor = 32
	movwf	PORTB		; escreve nos leds.
vetec:	
	btfss	PORTA,S0	; Se porta.0 = 1 (S0 aberta) pula.
	goto	S0fech		; portA.0 = 0. (S0 fech) desvia
S0Ab:	
	btfsc	PORTA,S1	; S0 aberta, testa S1. Se porta.1 = 0 (S1 fech) pula.
	goto	vetec		; S1 tambem aberta, repete espera de teclas.
S1fech:				; chegou aqui porque S1 esta fech.
	incf	valor		; valor = valor + 1
	btfss	valor,6		; Se valor.bit6=1 passou do 63.
	goto   	S1ok		; aceitou S1
	decf	valor		; nao aceitou, volta
S1ok:
	movf	valor,w		; copia valor em W e
	movwf	PORTB		; escreve nos leds.
	call	ms250		; ESPERA 250 MS ENTRE REPETICAO DE TECLAS.
				; EXPERIMENTE RODAR SEM ESTE DELAY.
				; (COLOQUE UM ; ANTES DO CALL.)
	goto	vetec		; volta para ver teclas novamente
S0fech:				; chegou aqui porque S0 esta fech.
	decf	valor		; valor = valor - 1
	btfss	valor,7		; Se valor.bit7=1, de 0 foi para 255
	goto	S0ok		; valor >= 0
	incf	valor		; volta ao normal.
S0ok:
	movf	valor,w		; copia valor em W e
	movwf	PORTB		; escreve nos leds.
	call	ms250		; ESPERA 250 MS ENTRE REPETICAO DE TECLAS.
				; EXPERIMENTE RODAR SEM ESTE DELAY.
				; (COLOQUE UM ; ANTES DO CALL.)
	goto	vetec		; volta para ver teclas novamente

;...............................Rotina de tempo de 250 ms.
ms250:
	movlw	250		;1
	movwf	tempo		;1
ms250a:
	movlw 	248           	;1
	movwf	dt1           	;1
ms250b:
	nop               	;1	
	decfsz 	dt1        	;1
	goto  	ms250b     	;2
	decfsz	tempo		;1
	goto 	ms250a       	;2
fim250:
	return

	END			; fim do programa fonte
