;====================================================================
;	Programa exemplo 5 para o livro "Microcontroladores PIC"
;
;	Objetivo: 		Testar leitura e escrita na eeprom
;	Funcionamento: 		posicao "valor" varia entre 0 e 63 (00111111)
;				e indica o endereco E o dado que sera gravado
;				Ex.: valor = 10 -> No endereco 10 grava o dado 10
;				Inicia com valor = 8 = 00001000
; 				- S0 -> valor = valor - 1, se valor > 0
; 				- S1 -> valor = valor + 1, se valor < 63
;				- S2 press, salva "valor+128" no endereco "valor"
;				- enquanto S3 press, le o dado escrito no endereco 
;				  "valor" e escreve nos leds.
;	17/09/98 	Leds agora acendem em 1, em oposicao ao do livro.
;			Adaptacao para a placa Piclab 3	
;=====================================================================
	list p=16F84		; processador 16F84

	__config B'0011111111110001' 
				; CP off, DATA off, WD off, PWT on, XT


	radix	dec		; define padrao DECIMAL para valores sem identificacao
	include	<P16F84.INC>	; anexa o arquivo que contem as definicoes do 16F84
;.....................................................................................
;				Tabela de definicoes de RAM e constantes.
W	equ	0		; referencia de W
S0	equ	0		; tecla S0 esta ligada a RA0 (PORTA bit 0)
S1	equ	1		; tecla S1 esta ligada a RA1 (PORTA bit 1)
S2	equ	2		; tecla S0 esta ligada a RA2 (PORTA bit 2)
S3	equ	3		; tecla S1 esta ligada a RA3 (PORTA bit 3)
tempo	equ	0CH		; define variavel auxiliar tempo na RAM 0CH. (primeira posicao)
dt1	equ	0DH		; variavel auxiliar para tempo
X	equ	0EH		
Y	equ	0FH
valor   equ	10H		; valor ajustado por S0 e S1
lido	equ	11H		; dado lido da eeprom
gravado equ	12H		; dado gravado na eeprom


;----------------------------- Memoria de Programa -----------------------------------

	org	0		; define inicio do trecho a seguir em 000

	goto	inicio		; desvia para o programa principal onde inicia pelo 
				; ajuste dos registros de controle.
				; Este desvio tem por objetivo deixar o endereco 004
				; para a rotina de interrupcao.

;.............................. INICIO DA ROTINA DE INTERRUPCAO ............................
	org	4		

	nop			; primeira instrucao da interrupcao.
	retfie			; retorna da interrupcao.





;.............................. INICIO DO PROGRAMA .........................................

inicio:			; este endereco e' o imediatamente apos o fim da interrupcao
				; AQUI ESTAMOS NO BANCO 0
	movlw	B'00000000'	; W = 0
	movwf	INTCON	; INTCON = 0. Nenhuma interr habilitada.

	clrf	PORTA		;inicializa portas A e B
	clrf	PORTB

	bsf	STATUS,RP0	; Seleciona banco 1 para option e tris

	movlw	B'11111111'	; W = 255
	movwf	TRISA		; TRISA = ffH. PortA toda entrada

	movlw	B'00000000'	; W = 0
	movwf	TRISB		; TRISB = 0. PortB toda saida

	movlw	B'11011111'	;
	movwf	OPTION_REG	; Registro OPTION ajusta timer 0 clock interno
				; e prescaler p/ Watch Dog (nao usamos)
	bcf	STATUS,RP0	; volto ao banco 0. Padrao do reset

;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
principal:			;inicio do programa principal.

	movlw	32		; W = 00100000
	movwf	valor		; valor = 32
	movwf	PORTB		; escreve nos leds.

vetec:	
	btfss	PORTA,S0	; Se porta.0 = 1 (S0 aberta) pula.
	goto	S0fech		; portA.0 = 0. (S0 fech) desvia
S0Ab:	
	btfsc	PORTA,S1	; S0 aberta, testa S1. Se porta.1 = 0 (S1 fech) pula.
	goto	veS2S3		; S1 tambem aberta, ve se S2 ou S3 estao press

		;### ................. Ajuste de endereco/valor por S0 e S1
S1fech:				; chegou aqui porque S1 esta fech.
	incf	valor		; valor = valor + 1
	btfss	valor,6		; Se valor.bit6=1 passou do 63.
	goto   	S1ok		; aceitou S1
	decf	valor		; nao aceitou, volta
S1ok:
	movf	valor,W		; copia valor em W e
	movwf	PORTB		; escreve nos leds.
	call	ms250
	goto	vetec

S0fech:				; chegou aqui porque S0 esta fech.
	decf	valor		; valor = valor - 1
	btfss	valor,7		; Se valor.bit7=1, de 0 foi para 255
	goto	S0ok		; valor >= 0
	incf	valor		; volta ao normal.
S0ok:
	movf	valor,W		; copia valor em W e
	movwf	PORTB		; escreve nos leds.
	call	ms250
	goto	vetec
			;### Ve se Salva/le
veS2S3:
	btfss	PORTA,S2	; ve se S2 (salvar) esta solta (=1)
	goto	Salvar		; nao esta, vai salvar

	btfss	PORTA,S3	; ve se S3 (le) esta solta (=1)
	goto 	Ler		; naoesta, vai ler o dado

	goto	vetec		; repete do inicio

Salvar:					; somar 128 ao valor (bit 7 = 1)
	bcf	STATUS,RP0	
	movf	valor,W			; W = valor
	movwf	EEADR			; endereco = W = valor
	iorlw	80H			; W = W OR 80H = valor OR 80H (seta bit 7)
	movwf	EEDATA		; dado = valor com bit 7 = 1
	bsf	STATUS,RP0
	bsf	EECON1,WREN	; habilita escrita. Interr ja desab

	movlw	55H
	movwf	EECON2	
	movlw	0AAH			; CONSTANTES HEXA INICIADAS COM LETRAS
	movwf	EECON2		; DEVEM TER UM '0' ANTES.
	bsf	EECON1,WR		; inicia processo de escrita
espera2:
	btfss	EECON1,EEIF		; ve se ja finalizou escrita
	goto 	espera2		; EEIF = 0. ainda nao escreveu
	bcf	EECON1,WREN	; desabilita escrita na eeprom
	bcf	EECON1,EEIF
	bcf	STATUS,RP0	
	goto	vetec

Ler:
	bcf	STATUS,RP0		; banco 0
	movf	valor,W			; W = valor
	movwf	EEADR			; endereco = valor
	bsf	STATUS,RP0		; banco 1
	bsf	EECON1,RD		; inicia leitura
	bcf	STATUS,RP0		; banco 0
	movf	EEDATA,W		; move o dado lido para W
	movwf	lido			; salva em lido
	movwf	PORTB			; escreve no port B o dado lido

espera1:
	call	ms250		; perde tempo
	btfss	PORTA,3		; ve se S3 esta fechada.
	goto 	espera1		; esta, repete leitura
	
	movf	valor,W		; copia valor em W e
	movwf	PORTB		; Volta a mostrar o ajuste de endereco
	goto 	vetec	

;...............................Rotina de tempo de 250 ms.
ms250:
	movlw	250		;1
	movwf	tempo		;1

ms250a:
	movlw 	248           	;1
	movwf	dt1           	;1

ms250b:
	nop                   		;1	
	decfsz 	dt1        	;1
	goto  	ms250b     	;2

	decfsz	tempo		;1
	goto 	ms250a       	;2

	return
;......................................................................

; Define os dados na EEProm nas 64 posicoes

	org	2100H

	de	255,255,255,255,255,255,255,255
	de	255,255,255,255,255,255,255,255
	de	255,255,255,255,255,255,255,255
	de	255,255,255,255,255,255,255,255
	de	255,255,255,255,255,255,255,255
	de	255,255,255,255,255,255,255,255
	de	255,255,255,255,255,255,255,255
	de	255,255,255,255,255,255,255,255

	END			; fim do programa fonte
