;=======================================================================
;	Programa exemplo 7 para o livro "Microcontroladores PIC"
;
;	Objetivo: 	Estudar o watch dog
;	Funcionamento: 	Se ficar mais de 2 segundos sem pressionar
;			S0 o chip sera resetado
;	IMPORTANTE:	O WATCH DOG NAO E' LIGADO NO SOFTWARE.
;			DEVERA SER LIGADO MENU DE GRAVACAO DO PICSTART+
;	Agosto/97
;			Adaptado para a placa Piclab 3
;=======================================================================
	list p=16F84		; processador 16F84

	__config B'0011111111110001' 
				; CP off, DATA off, WD off, PWT on, XT

	radix	dec		; define padrao DECIMAL para valores sem identificacao
	include	<P16F84.INC>	; anexa o arquivo que contem as definicoes do 16F84
;..........................................................................
;		Tabela de definicoes de RAM e constantes.
W	equ	0		; referencia a W
Led7	equ	7		; led 7 esta em RB7
S0	equ	0		; tecla S0 em PORTA bit 0
tempo	equ	0CH		; define variavel auxiliar tempo na RAM 0CH. (primeira posicao)
X	equ	0DH
Y	equ	0EH

;----------------------------- Memoria de Programa ------------------------

	org	0		; define inicio do trecho a seguir em 000

	goto	inicio		; desvia para o programa principal onde inicia pelo 
				; ajuste dos registros de controle.
				; Este desvio tem por objetivo deixar o endereco 004
				; para a rotina de interrupcao.
;.............................. INICIO DA ROTINA DE INTERRUPCAO ..........
	org	4		

	nop			; primeira instrucao da interrupcao.
	retfie			; retorna da interrupcao.

;.............................. INICIO DO PROGRAMA ......................

inicio:				; este endereco e' o imediatamente apos o fim da interrupcao

	movlw	B'00000000'	; W = 0
	movwf	INTCON		; INTCON = 0. Nenhuma interr habilitada.

	bsf	STATUS,RP0	; Seleciona banco 1 para option e tris
	movlw	B'11011111'	;
	movwf	OPTION_REG	; Registro OPTION ajusta timer 0 clock interno
				; e prescaler p/ Watch Dog (nao usamos)
	movlw	B'11111111'	; W = 255
	movwf	TRISA		; TRISA = ffH. PortA toda entrada
	movlw	B'00000000'	; W = 0
	movwf	TRISB		; TRISB = 0. PortB toda saida

	bcf	STATUS,RP0	; volto ao banco 0. Padrao do reset



;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
principal:			;inicio do programa principal.

	movlw	255
	movwf	PORTB		; todos os leds acesos por +- 1s
	movlw	10
	movwf	Y		; faco Y ir de 10 a 0.
seg1:
	call	ms100		; +- 100 ms
	decfsz	Y		; Y = Y - 1. se Y = 0 pula o goto
	goto	seg1
	clrf	PORTB		; apaga todos os leds
repete:
	bcf	PORTB,Led7 	; fazendo PORTB pino 7 = 0 apaga o led 7
	call	ms100		; espera 100 ms
	bsf	PORTB,Led7 	; fazendo PORTB pino 7 = 1 acende o led 7
	call	ms100		; espera 100 ms

	btfss	PORTA,S0	; Vejo se S0 = 1 (solta) 
	clrwdt			; S0 esta press (=0), clear no wdt
	goto 	repete

;...............................Rotina de tempo de 100 ms.
ms100:				
	movlw	100		; apenas troquei o 250 por 100.	
	movwf	tempo		; o tempo deve ser +- 100 ms

ms100a:
	movlw 	248           	
	movwf	X           	

ms100b:			
	nop               	
	decfsz 	X        	
	goto  	ms100b     	

	decfsz	tempo		
	goto 	ms100a       	
				
	return			

	END			; Fim do programa fonte

