;=====================================================================
;	Programa exemplo do livro Microcontroladores PIC
;
;	Objetivo: 		Mostrar o uso do modulo A/D
;	Funcionamento: 		Conforme variamos o potenciometro temos nos
;				leds ligados ao PORTB o valor digital convertido.
;				Espero o fim por varredura em ADIF (ou GO/DONE)
;	Agosto/97
;			Adaptado a placa PicLab 3
;=================================================================== 
	list p=16C711		; processador 16C711

	__config B'0011111111110001' 
				; CP off, DATA off, WD off, PWT on, XT

	radix	dec		; define padrao DECIMAL para valores sem identificacao
	include	<P16C711.INC>	; anexa o arquivo que contem as definicoes do 16C711
;.....................................................................................
;				Tabela de definicoes de RAM e constantes.

tempo	equ	0CH		; define variavel auxiliar tempo na RAM 0CH. (primeira posicao)
dt1	equ	0DH		; variavel auxiliar para tempo
valor	equ	0EH		; onde salvo o valor lido do ADRES

;----------------------------- Memoria de Programa -----------------------------------

	org	0		; define inicio do trecho a seguir em 000
				; nao reservei area para interrupcoes porque nao 
				; serao usadas

inicio:								
	movlw	B'00000000'	; W = 0
	movwf	INTCON		; INTCON = 0. Nenhuma interr habilitada.

	clrf	PORTA		;inicializa portas A e B
	clrf	PORTB

	bsf	STATUS,RP0	; Seleciona banco 1 para OPTION,  TRIS e ADCON1

	movlw	B'11111111'	; W = 255
	movwf	TRISA		; TRISA = ffH. PortA toda entrada

	movlw	B'00000000'	; W = 0
	movwf	TRISB		; TRISB = 0. PortB toda saida

	movlw	B'00000000'	; W = 0
	movwf	OPTION_REG	; Registro OPTION = W.

	movlw	B'00000010'	; W = 2
	movwf	ADCON1		; Registro ADCON1 = 2. Canais 0 e 1 A/D, demais entrada digital

	bcf	STATUS,RP0	; volto ao banco 0. Padrao do reset e onde esta ADCON0

	movlw	B'01000001'
	movwf	ADCON0		; Clock do A/D Fosc/8, canal 0, modulo ligado.

;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
principal:			;inicio do programa principal.

us10:				;Perde 10 us
	movlw	3		; 1 ciclo				(1)			
	movwf	dt1		; 1 ciclo				(2)
esp:
	decfsz	dt1		; 1 ciclo se > 0, senao 2	(3)
	goto 	esp		; 2 ciclos se >0, senao 0	(4)
								
		; Explicacao do loop:
		;
		;	Perde 1 us em (1)	
		; 	Perde 1 us em (2)	
		; 	Perde 1 us em (3)	porque nao resultou > 0 (3 -1 = 2)
		; 	Perde 2 us em (4)	 
		; 	Perde 1 us em (3)	
		; 	Perde 2 us em (4)	porque nao resultou > 0 (2 -1 = 1)
		; 	Perde 2 us em (3)	porque resultou = 0 (1 -1 = 0). o goto  ignorado


	bsf	ADCON0,GO	; inicia a conversao
espera:
	btfss	ADCON0,ADIF	; se ADIF = 1 pula prox instrucao 
	goto	espera		; ADIF = 0. Espera ir a 1
	bcf	ADCON0,ADIF	; Reseta flag de interrupcao

	movf	ADRES,0		; W = ADRES
	movwf	valor		; valor = ADRES e
	movwf	PORTB 		; escreve no PORTB.

	call 	ms250		; espera 250 ms antes de repetir
				; evita que os leds fiquem instaveis devido a ruido e imprecisao 
				; no potenciometro.
	goto    principal

;...............................Rotina de tempo de 250 ms.
ms250:
	movlw	250		;1
	movwf	tempo		;1
ms250a:
	movlw 	249           	;1
	movwf	dt1           	;1
ms250b:
	nop               	;1	
	decfsz 	dt1        	;1
	goto  	ms250b     	;2
	decfsz	tempo		;1
	goto 	ms250a       	;2
	return

	END			; fim do programa fonte
